// ======================================================================
//
// Extents.h
// jeff grills
//
// copyright 1997 origin systems inc
//
// -qq- possibly add scaling of extent mesh
// -qq- possible add proximity to collide() functions
//
// ======================================================================

#ifndef EXTENTS_H
#define EXTENTS_H

// ======================================================================

class Appear;
class Camera;
class Iff;
class Xform;

#include "Vector.h"

// ======================================================================

class Extents
{
private:

	class Node
	{
	public:
		
		enum Type
		{
			split,
			full,
			empty
		};

	private:

		Type        type;
		Vector      normal;
		real        planeD;
		int         region;

		Vector     *vertex;
		int         vertexCount;

		Node       *front;
		Node       *back;

	private:

		Node(void);
		Node(const Node &);
		Node &operator =(const Node &);

	public:

		explicit Node(Iff &iff);
		~Node(void);
	
		int           getRegion(void) const;
		const Vector &getNormal(void) const;

		bool  collide(const Vector &point, const Node **node, const Node **split) const;
		bool  collide(const Vector &start, const Vector &stop, Vector *intersectLocation, const Node **node, const Node **split) const;
		
		void  render(void) const;
	};

private:

	int           users;

	Vector        sphereCenter;
	real          sphereRadius;

	Node         *root;

	int           regions;
	Vector      **regionVertex;
	int8         *regionVertexCount;
	int8        **regionVertexIndex;

private:

	void init(void);

	Extents(void);
	Extents(const Extents &);
	Extents &operator=(const Extents &);

public:

	static void addUser(Extents *extents);
	static void removeUser(Extents *extents);
	
	Extents(const Vector &newCenter, real newRadius);
	explicit Extents(Iff &iff);
	~Extents(void);

	const Vector &getSphereCenter(void) const;
	void          setSphereCenter(const Vector& newCenter);

	real          getSphereRadius(void) const;
	void          setSphereRadius(real newRadius);

	bool collideSpheres(const Vector &center, real radius) const;
	bool collideSpheres(const Vector &start, const Vector &end, real radius, Vector *result=NULL) const;
	
	bool collide(const Vector &point, int *regionNumber=NULL, Vector *normal=NULL) const;
	bool collide(const Vector &start, const Vector &end, Vector *result, int *regionNumber=NULL, Vector *normal=NULL) const;

	void render(void) const;
};

// ======================================================================

inline int Extents::Node::getRegion(void) const
{
	return region;
}

// ----------------------------------------------------------------------

inline const Vector &Extents::Node::getNormal(void) const
{
	return normal;
}

// ======================================================================

inline void Extents::addUser(Extents *extents)
{
	if (extents)
		extents->users++;
}

// ----------------------------------------------------------------------

inline void Extents::removeUser(Extents *extents)
{
	if (extents && --extents->users == 0)
		delete extents;
}

// ----------------------------------------------------------------------

inline const Vector &Extents::getSphereCenter(void) const
{
	return sphereCenter;
}

// ----------------------------------------------------------------------

inline real Extents::getSphereRadius(void) const
{
	return sphereRadius;
}

// ----------------------------------------------------------------------

inline void Extents::setSphereCenter(const Vector& newCenter)
{
	sphereCenter = newCenter;
}

// ----------------------------------------------------------------------

inline void Extents::setSphereRadius(real newRadius)
{
	sphereRadius = newRadius;
}

// ======================================================================

#endif
